# This file is part of the ElastiSim software, modified by BSC.
#
# Copyright (c) 2022, Technical University of Darmstadt, Germany
# Copyright 2021-2023, Barcelona Supercomputing Center (BSC), Spain
# This software was partially supported by the EuroHPC-funded project ADMIRE
#   (Project ID: 956748, https://www.admire-eurohpc.eu).
#
# This software may be modified and distributed under the terms of the 3-Clause
# BSD License. See the LICENSE file in the base directory for details.

from typing import Any
from os import path
from elastisim_python import JobState, JobType, NodeState, pass_algorithm, Job, Node

filename = "data/output/breaks.txt"

def write_integer_to_file(filename, number):
    with open(filename, 'w') as file:
        file.write(str(number))

def read_integer_from_file(filename):
    if not path.isfile(filename):
        return 0
    with open(filename, 'r') as file:
        number = int(file.read())
        return number

def update_integer_in_file(filename, update_value):
    number = read_integer_from_file(filename)
    updated_number = number + update_value
    write_integer_to_file(filename, updated_number)

def schedule(jobs: list[Job], nodes: list[Node], system: dict[str, Any]) -> None:
    time = system['time']
    pending_jobs = [job for job in jobs if job.state == JobState.PENDING]
    running_jobs = [job for job in jobs if job.state == JobState.RUNNING]

    free_nodes = [node for node in nodes if node.state == NodeState.FREE]

    shrink = False
    expand = False
    schedule = False
    # Check if we can allocate more jobs (all jobs are malleable)
    high_bw = 0
    low_bw = 0
    #We should try to have 1 high bw job and 1 low job
    for job in pending_jobs:
        requestedBw = float(job.attributes["BW"])
        if not free_nodes:
            break
        is_minimum = True
        if (schedule):
            if (high_bw >= low_bw):
                # Check if there is a job with lower requirements
                for other_job in pending_jobs:
                    other_requestedBw = float(other_job.attributes["BW"])
                    if (requestedBw > other_requestedBw):
                        is_minimum = False
                        break
            if (is_minimum):
                #Run the job if possible
                if (high_bw >= low_bw):
                    low_bw = low_bw + 1
                else:
                    high_bw = high_bw + 1
                num_nodes_to_assign = min(len(free_nodes), 3 )#job.num_nodes_max)
                if num_nodes_to_assign >= job.num_nodes_min:
                    job.assign(free_nodes[:num_nodes_to_assign])
                    del free_nodes[:num_nodes_to_assign]
        else:
            num_nodes_to_assign = min(len(free_nodes), 3 ) #job.num_nodes_max)
            if num_nodes_to_assign >= job.num_nodes_min:
                job.assign(free_nodes[:num_nodes_to_assign])
                del free_nodes[:num_nodes_to_assign]

    for job in running_jobs:
        requestedBw = float(job.attributes["BW"])
        bytes_write = float(job.completed_phases)*float(job.arguments["bytes_write"])
        if (time-job.start_time > 10) :
            bw = (bytes_write/(1024*1024*1024))/(time-job.start_time)       
            if (bw > (requestedBw+requestedBw*0.1)):
                num_nodes_to_shrink = 1
                if (num_nodes_to_shrink < len(job.assigned_nodes)) :
                    if (shrink):
                       # print("BW Obtained : "+str(bw)+ " Requested "+str(requestedBw))
                       # print("Shrink : "+str(len(job.assigned_nodes))+"Job"+str(job.identifier))
                        node = job.assigned_nodes[len(job.assigned_nodes)-1]
                        job.remove(node)
            else:
                if (bw < (requestedBw-requestedBw*0.1)):
                    update_integer_in_file(filename, 1)
                    if (job.num_nodes_max > len(job.assigned_nodes)):
                        num_nodes_to_expand = min(len(free_nodes), 1)
                        if num_nodes_to_expand > 0:
                            if (expand):
                               # print("Expand: "+str(job.identifier) + " From " +str(len(job.assigned_nodes)))
                                job.assign(free_nodes[:num_nodes_to_expand])
                                del free_nodes[:num_nodes_to_expand]
       
if __name__ == '__main__':
    url = 'ipc:///tmp/elastisim.ipc'
    pass_algorithm(schedule, url)
